#include <stdlib.h>
#include "rccpatch.h"

#ifdef HAVE_LIBRCC
# include <librcc.h>
#endif /* HAVE_LIBRCC */


#ifdef HAVE_LIBRCC
# define ID3_CLASS 0
# define ID3V2_CLASS 1
# define UTF_CLASS 2
# define OUT_CLASS 3
static rcc_class classes[] = {
    { "id3", RCC_CLASS_STANDARD, NULL, NULL, "ID3 Encoding", 0 },
    { "id3v2", RCC_CLASS_STANDARD, "id3", NULL, "ID3 v.2 Encoding", 0 },
    { "utf", RCC_CLASS_KNOWN, "UTF-8", NULL, "Unicode Encoding", 0},
    { "out", RCC_CLASS_TRANSLATE_LOCALE, "LC_CTYPE", NULL, "Output Encoding", 0 },
    { NULL, RCC_CLASS_STANDARD, NULL, NULL, NULL, 0 }
};

static int rcc_initialized = 0;

static rcc_context ctx = NULL;
#endif /* HAVE_LIBRCC */


void rccPatchFree() {
#ifdef HAVE_LIBRCC
    if (rcc_initialized) {
       rccFree();
       rcc_initialized = 0;
    }
#endif /* HAVE_LIBRCC */
}

void rccPatchInit() {
#ifdef HAVE_LIBRCC
    if (rcc_initialized) return;
    rccInit();
    rccInitDefaultContext(NULL, 0, 0, classes, 0);
    rccLoad(NULL, "xmms");
    rccInitDb4(NULL, NULL, 0);
    rcc_initialized = 1;
#endif /* HAVE_LIBRCC */
}

void rccPatchSetContext(void *newctx) {
#ifdef HAVE_LIBRCC
    if (newctx) {
	ctx = (rcc_context)newctx;
	rcc_initialized = 1;
    }
#endif /* HAVE_LIBRCC */
}

static void rccPatchTryInit() {
#ifdef HAVE_LIBRCC
    if (!rcc_initialized) {
	rccPatchInit();
	if (rcc_initialized) atexit(rccPatchFree);	
    }
#endif /* HAVE_LIBRCC */
}


id3_utf8_t *rccPatchLatin2UTF(id3_latin1_t *str) {
#ifdef HAVE_LIBRCC
    rccPatchTryInit();

    return rccRecode(ctx, ID3_CLASS, UTF_CLASS, str);
#else
    return  NULL;
#endif /* HAVE_LIBRCC */
}

id3_latin1_t *rccPatchUTF2Latin(id3_utf8_t *str) {
#ifdef HAVE_LIBRCC
    rccPatchTryInit();

    return rccRecode(ctx, UTF_CLASS, ID3_CLASS, str);
#else
    return  NULL;
#endif /* HAVE_LIBRCC */
}

id3_latin1_t *rccPatchUTF2Out(id3_utf8_t *str) {
#ifdef HAVE_LIBRCC
    rccPatchTryInit();

    return rccRecode(ctx, UTF_CLASS, OUT_CLASS, str);
#else
    return  NULL;
#endif /* HAVE_LIBRCC */
}

